
#include "csources/svm.h"
#include "csources/svm_utils.h"
#include "csources/svm_params.h"
#include "csources/kernel_func.h"
#include "csources/utils.h"
#include "csources/DiskUtils.h"
#include "csources/Data.h"
#include "csources/MXData.h"

#include <cstring>

using namespace std;

void mexFunction( int nlhs, mxArray *plhs[],
             int nrhs, const mxArray *prhs[] )
{
	if((nlhs == 2) && (nrhs >= 7)) {

		const mxArray* mx_ALPHA		= prhs[0];
		const mxArray* mx_B 		= prhs[1]; 
		const mxArray* mx_SVs 		= prhs[2];
		const mxArray* mx_UCL		= prhs[3];
		const mxArray* mx_kernel_type 	= prhs[4];
		const mxArray* mx_params 	= prhs[5];
		const mxArray* mx_X 		= prhs[6];
		const mxArray* mx_nf		= (nrhs > 7)  ? prhs[7]  : NULL;
		const mxArray* mx_test_index    = (nrhs > 8)  ? prhs[8]  : NULL;
		const mxArray* mx_save_memory   = (nrhs > 9)  ? prhs[9]  : NULL;
		const mxArray* mx_mask		= (nrhs > 10) ? prhs[10] : NULL;
		const mxArray* mx_sv_indizes    = (nrhs > 11) ? prhs[11] : NULL;

		int save_memory = (!mx_save_memory) ? 0 : (int) *mxGetPr(mx_save_memory);

		int alpha_columns = mxGetN(mx_ALPHA);
		int alpha_rows = mxGetM(mx_ALPHA);
		double**pALPHA = (double**) pointer_array(alpha_columns);

		pALPHA[0] = mxGetPr (mx_ALPHA);
		for(int i=1; i < alpha_columns; i++) pALPHA[i] = pALPHA[0] + i*alpha_rows;

		double** ALPHA = m_transpose(pALPHA, alpha_columns, alpha_rows);
		free_array(pALPHA);

		int B_size = mxGetN(mx_B) * mxGetM(mx_B);	
		double* B  = double_array(B_size);
		memcpy (B, mxGetPr (mx_B), B_size*sizeof(double));

		DataDesc* X_train = MXDataDesc::Create(mx_SVs, save_memory);

		int X_train_rows    = X_train->ssize();
		int X_train_columns = X_train->size();

		int* sv_indizes     = NULL;
		int sv_indizes_size = (mx_sv_indizes != NULL) ? mxGetM(mx_sv_indizes)*mxGetN(mx_sv_indizes) : 0;	
		if (sv_indizes_size > 0){
			sv_indizes = int_array(sv_indizes_size);

			if (mxGetClassID (mx_sv_indizes) == mxINT32_CLASS) 
				memcpy(sv_indizes, (int*) mxGetData(mx_sv_indizes), sv_indizes_size*4);
			else double2int(sv_indizes, mxGetPr(mx_sv_indizes), sv_indizes_size);

			issub_ip(sv_indizes, 1, sv_indizes_size); // need zero based indices
		}
		else { // consider the entire data
			sv_indizes_size = X_train->size();
			sv_indizes = int_array(sv_indizes_size);
			for (int i=0;i < sv_indizes_size; i++) sv_indizes[i] = i;
		}

		DataDesc* SVs = new SubsetDataDesc (sv_indizes, sv_indizes_size, X_train);

		int SVs_rows    = SVs->ssize();
		int SVs_columns = SVs->size();

		// Warning -> switch rows and columns !
		int UCL_rows = mxGetN (mx_UCL);
		int ccount   = mxGetM(mx_UCL);
		
		double** pUCL = (double**) pointer_array(UCL_rows);
		pUCL[0] = mxGetPr(mx_UCL);
		for (int i=1; i < UCL_rows; i++) pUCL[i] = pUCL[0] + i*ccount;

		double** UCL = m_create(UCL_rows, ccount);
		m_copy (UCL,pUCL, UCL_rows, ccount);
		free_array(pUCL);

		ccount = (ccount > 2) ? ccount : 1;

		PRINTF("Number of classifiers: %i\n", ccount);

		int kernel_type = (int) *mxGetPr (mx_kernel_type);

		int params_len = mxGetN(mx_params)*mxGetM(mx_params);
		double* params = double_array(params_len); 
		memcpy (params, mxGetPr (mx_params), params_len* sizeof(double));

		//-----------------

		DataDesc* X = MXDataDesc::Create(mx_X, save_memory);

		if(SVs_rows != X->ssize()) {
			PRINTF("Number of SV dimensions does not equal number of test dimensions !!!\n");
			return;
		}

		int* test_index     = NULL;
		int test_index_size = (mx_test_index != NULL) ? mxGetM(mx_test_index)*mxGetN(mx_test_index) : 0;

		if (test_index_size > 0) {
			test_index = int_array(test_index_size);

			if (mxGetClassID (mx_test_index) == mxINT32_CLASS) memcpy(test_index, (int*) mxGetData(mx_test_index), test_index_size*4);
			else double2int(test_index, mxGetPr(mx_test_index), test_index_size);

			issub_ip(test_index, 1, test_index_size); // need zero based indices
		}
		else { // consider the entire data
			test_index_size = X->size();
			test_index = int_array(test_index_size);
			for (int i=0;i < test_index_size; i++) test_index[i] = i;
		}

		DataDesc* X_test = new SubsetDataDesc (test_index, test_index_size, X);

		int X_rows    = X_test->ssize();
		int X_columns = X_test->size();

		PRINTF("Number of samples to be classified: %i\n", X_columns);

		int* mask     = NULL;
		int mask_size = (mx_mask != NULL) ? mxGetM(mx_mask)*mxGetN(mx_mask) : 0;

		if (mask_size > 0) {
			PRINTF("Using mask of size %i  ...\n", mask_size);

			mask = int_array(mask_size);

			double* pMask = mxGetPr(mx_mask);

			double2int(mask, pMask , mask_size);
			issub(mask, mask, 1, mask_size);
		}

		//-----------------

		double nf = (mx_nf != NULL)  ? ((!mxIsEmpty(mx_nf)) ?  *mxGetPr(mx_nf) : 1.0)  : 1.0;

		Kernel* K = Kernel::create_kernel(kernel_type, params, params_len);
		K->SetNormFactor(nf);	

		if (K==NULL) {
			PRINTF("Could not interpret kernel parameters ... bailing out.\n");
			return;
		}

		if (mask != NULL) X_test = new DimSelectDataDesc(mask, mask_size, X_test);

		double** DECISION = m_create (X_columns, ccount);
		double* labels = NULL;

		svm_classify (&DECISION, &labels, ALPHA, B, SVs, UCL, UCL_rows, ccount, K, X_test);

		delete K;

		mxArray* mx_DECISION = mxCreateNumericMatrix (ccount, X_columns, mxDOUBLE_CLASS, mxREAL);
		m_to_vec(mxGetPr(mx_DECISION), DECISION, X_columns, ccount);
		m_free(DECISION);

		mxArray* mx_labels = mxCreateNumericMatrix (1, X_columns, mxDOUBLE_CLASS, mxREAL);
		memcpy(mxGetPr(mx_labels), labels, X_columns*sizeof(double));
		free_array(labels);

		plhs [0] = mx_DECISION;
		plhs [1] = mx_labels;

		m_free(ALPHA);

		if (mask != NULL) free_array(mask);

		delete X;
		delete X_test;

		delete X_train;
		delete SVs;

		m_free(UCL);
		free_array(B);
		PRINTF("finished ...\n");
	}
	else mexPrintf("Wrong number of input/output parameters (%i/%i). \n", nrhs, nlhs);
}
