%
%function [decision, labels] = MaxMinOverClassify (alpha, b, SVs, ucl, kernel_type, k_params, X, nf, test_index, save_memory, mask);
%
%Perform classification on given data with separation hyperplane obtained before from TrainMaxMinOver or CVMaxMinOver.
%
% typical usage:
%     [decision, labels] = MaxMinOverClassify (alpha, b, SVs, ucl, kernel_type, k_params, X);
%
% if the hyper-parameters of the kernel have been determined by CVMaxMinOver use:
%     [decision, labels] = MaxMinOverClassify (alpha, b, SVs, ucl, kernel_type, k_params, X, nf);
%
% See also: http://www.inb.uni-luebeck.de/tools-demos/maxminover
%
%Input:
% alpha        : weights of the support vectors (for each two class problem)
% b            : threshold (for each two class problem)
% SVs          : support vectors
% ucl          : class assignment for each two class problem (see
%                TrainMaxMinOver for more information)
% kernel_type  : 0 -> linear, 1 -> RBF, 2 -> polynomial
% k_params     : specific kernel parameters (same as k_params for TrainMaxMinOver)
% 	       linear     : []
%	       rbf        : [gamma]
%	       polynomial : [degree, k]
%
% X           : data to be classified (either matrix or name of ascii file containing the test data, one sample per row)
% nf 	      : normalization factor (only required if CVMaxMinOver was used in order to
%               determine the hyper-parameters, default: 1.0)
% test_index  : allows to specify a subset of X that should be tested (optional)
% save_memory : determines if classification should be performed as memory efficient as possible (default 0->off)
% mask        : matrix containing the indices of the dimensions to be used for each two class classifier (optional)
%
%Return Values:
%   decision : decision values for each two class problem of data classified (threshold to be added) 
%   labels   : resulting labels if threshold b is used 
%
function [decision, labels] = MaxMinOverClassify (alpha, b, SVs, ucl, kernel_type, k_params, X, nf, test_index, save_memory, mask);
