#include "mex.h"
#include "csources/svm.h"
#include "csources/svm_utils.h"
#include "csources/svm_params.h"
#include "csources/kernel_func.h"
#include "csources/utils.h"
#include "csources/DiskUtils.h"
#include "csources/Data.h"
#include "csources/MXData.h"

#include <cstring>

using namespace std;

void mexFunction( int nlhs, mxArray *plhs[],
             int nrhs, const mxArray *prhs[] )
{
	if((nlhs == 1) && (nrhs >= 6)) {

		const mxArray* mx_ALPHA		= prhs[0];
		const mxArray* mx_B 		= prhs[1];
		const mxArray* mx_SVs 		= prhs[2];
		const mxArray* mx_UCL		= prhs[3];
		const mxArray* mx_kernel_type 	= prhs[4];
		const mxArray* mx_params 	= prhs[5];
		const mxArray* mx_nf		= (nrhs > 6)  ? prhs[6]  : NULL;
		const mxArray* mx_save_memory   = (nrhs > 7)  ? prhs[7]  : NULL;

		int save_memory = (mx_save_memory != NULL)  ? ((!mxIsEmpty(mx_save_memory)) ? *mxGetPr(mx_save_memory) : 0)  : 0;

		DataDesc* SVs = MXDataDesc::Create(mx_SVs, save_memory);

		int alpha_columns = mxGetN(mx_ALPHA);
		int alpha_rows    = mxGetM(mx_ALPHA);
		double**pALPHA    = (double**) pointer_array(alpha_columns);

		pALPHA[0] = mxGetPr (mx_ALPHA);
		for(int i=1; i < alpha_columns; i++) pALPHA[i] = pALPHA[0] + i*alpha_rows;

		double** ALPHA = m_transpose(pALPHA, alpha_columns, alpha_rows);

		int B_size = mxGetN(mx_B) * mxGetM(mx_B);	
		double* B  = double_array(B_size);
		memcpy (B, mxGetPr (mx_B), B_size*sizeof(double));

		// Warning -> switch rows and columns!
		int UCL_rows = mxGetN(mx_UCL);
		int ccount   = mxGetM(mx_UCL);
		
		double** pUCL = (double**) pointer_array(UCL_rows);
		pUCL[0]       = mxGetPr(mx_UCL);
		for (int i=1; i < UCL_rows; i++) pUCL[i] = pUCL[0] + i*ccount;

		double** UCL = m_create(UCL_rows, ccount);
		m_copy (UCL, pUCL, UCL_rows, ccount);

		ccount = (ccount > 2) ? ccount : 1;

		PRINTF("Number of classifiers: %i\n", ccount);

		int kernel_type = (int) *mxGetPr (mx_kernel_type);

		int params_len = mxGetN(mx_params)*mxGetM(mx_params);
		double* params = double_array(params_len); 
		memcpy (params, mxGetPr (mx_params), params_len* sizeof(double));

		double nf = (mx_nf != NULL)  ? ((!mxIsEmpty(mx_nf)) ?  *mxGetPr(mx_nf) : 1.0)  : 1.0;

		Kernel* K = Kernel::create_kernel(kernel_type, params, params_len);	
		K->precompute(SVs);
		K->SetNormFactor(nf);

		if (K==NULL) {
			PRINTF("Could not interpret kernel parameters ... bailing out.\n");
			return;
		}

		double** RANK = m_create (ccount, SVs->ssize());

		int output=1;
		rank_features_margin (&RANK, ALPHA, B, SVs, ccount, K, output);
		delete K;

		mxArray* mx_RANK = mxCreateNumericMatrix (SVs->ssize(), ccount, mxDOUBLE_CLASS, mxREAL);
		m_to_vec(mxGetPr(mx_RANK), RANK, ccount, SVs->ssize());
		m_free(RANK);

		plhs [0] = mx_RANK;

		free_array(pALPHA);
		free_array(pUCL);
		m_free(ALPHA);

		delete SVs;

		m_free(UCL);
		free_array(B);

		PRINTF("finished ...\n");
	}
	else mexPrintf("Wrong number of input/output parameters (%i/%i). \n", nrhs, nlhs);
}

