#include "mex.h"
#include "csources/svm.h"
#include "csources/svm_utils.h"
#include "csources/svm_params.h"
#include "csources/utils.h"
#include "csources/DiskUtils.h"
#include "csources/kernel_func.h"
#include "csources/Data.h"
#include "csources/MXData.h"

#include <cstring>

using namespace std;

void mexFunction( int nlhs, mxArray *plhs[],
             int nrhs, const mxArray *prhs[] )
{
	if ((nlhs >= 5) && (nrhs >= 4)) {
	
		const mxArray* mx_X 		   = prhs[0];
		const mxArray* mx_c 		   = prhs[1];
		const mxArray* mx_l_params	   = prhs[2];
		const mxArray* mx_params 	   = prhs[3];

		const mxArray* mx_nf		   = (nrhs > 4) ? prhs[4] : NULL;// optinal normalization factor (might be obtained by CVMaxMinOver)
		const mxArray* mx_train_index      = (nrhs > 5) ? prhs[5] : NULL;// consider only a subset as training set
		const mxArray* mx_save_mem_flag    = (nrhs > 6) ? prhs[6] : NULL;// save memory switch

		const mxArray* mx_init_alpha       = (nrhs > 7) ? prhs[7] : NULL;// check if initial alpha values are provided
		const mxArray* mx_hash_func_params = (nrhs > 8) ? prhs[8] : NULL;// check if there are user defined universal hash parameters
		const mxArray* mx_info_params	   = (nrhs > 9) ? prhs[9] : NULL;// allows to control which information about the training process is provided 

		const mxArray* mx_mask 		   = (nrhs > 10) ? prhs[10] : NULL;
		const mxArray* mx_learn_index      = (nrhs > 11) ? prhs[11] : NULL;

		int return_sv_indizes = (nlhs > 5) ? 1:0;
		int return_info       = (nlhs > 6) ? 1:0;

		int l_params_length = mxGetN(mx_l_params)*mxGetM(mx_l_params);
		int params_length   = mxGetN(mx_params)  *mxGetM(mx_params);

		double*l_params	= double_array(l_params_length);
		memcpy (l_params, mxGetPr (mx_l_params), l_params_length*sizeof(double));

		double*params	= double_array(params_length);
		memcpy(params, mxGetPr (mx_params), params_length*sizeof(double));

		int save_memory = (mx_save_mem_flag != NULL)  ? ((!mxIsEmpty(mx_save_mem_flag)) ? *mxGetPr(mx_save_mem_flag) : 0)  : 0;

		ClassInfoDataDesc* X = MXDataDesc::Create(mx_X, mx_c, save_memory);

		int* train_index  = NULL;
		int tr_index_size = (mx_train_index != NULL) ? mxGetN(mx_train_index)*mxGetM(mx_train_index) : 0;

		if (tr_index_size) { // consider subset
			train_index = int_array(tr_index_size);

			if (mxGetClassID (mx_train_index) == mxINT32_CLASS) memcpy(train_index, (int*) mxGetData(mx_train_index), tr_index_size*4); 
			else double2int (train_index, mxGetPr(mx_train_index), tr_index_size);

			issub_ip(train_index, 1, tr_index_size); // need zero based indices
		}
		else { // consider the entire data
			tr_index_size = X->size();
			train_index = int_array(tr_index_size);
			for (int i=0;i < tr_index_size; i++) train_index[i] = i;
		}

		ClassInfoDataDesc* X_train = new SubsetClassInfoDataDesc (train_index, tr_index_size, X); 

		int* learn_index     = NULL;
		int learn_index_size = (mx_learn_index != NULL) ? mxGetN(mx_learn_index)*mxGetM(mx_learn_index) : 0;

		if (learn_index_size) { // consider subset
			learn_index = int_array(learn_index_size);

			if (mxGetClassID (mx_learn_index) == mxINT32_CLASS) memcpy(learn_index, (int*) mxGetData(mx_learn_index), learn_index_size*4); 
			else double2int (learn_index, mxGetPr(mx_learn_index), learn_index_size);

			issub_ip(learn_index, 1, learn_index_size); // need zero based indices
		}
		else { // consider the entire data
			learn_index_size = tr_index_size;
			learn_index      = int_array(learn_index_size);
			for (int i=0;i < learn_index_size; i++) learn_index[i] = i;
		}

		int X_rows    = X_train->ssize();
		int X_columns = X_train->size();
	
	        double*	c = double_array(X_columns);
		X_train->getClassInfo(c);		

		int* hash_func_params = NULL;
		int hash_func_params_length = (mx_hash_func_params != NULL) ? mxGetM(mx_hash_func_params)*mxGetN(mx_hash_func_params) : 0;

		if (hash_func_params_length > 0) { // check if there are user defined universal hash parameters
			if (mxGetClassID (mx_hash_func_params) == mxINT32_CLASS) hash_func_params=(int*)mxGetData(mx_hash_func_params);
			else mexPrintf ("Ignoring wrong hash func parameters. Type int32 required.\n"); 
		}

		double **ALPHA = NULL;
		double **init_ALPHA = NULL;
		int alpha_rows    = (mx_init_alpha != NULL) ? mxGetM(mx_init_alpha) : 0;
		int alpha_columns = (mx_init_alpha != NULL) ? mxGetN(mx_init_alpha) : 0;
		if (alpha_rows*alpha_columns > 0) {
			init_ALPHA    = (double**) pointer_array(alpha_columns);
			init_ALPHA[0] = mxGetPr(mx_init_alpha);
			for (int i=1; i < alpha_columns; i++) init_ALPHA[i] = init_ALPHA[0] + i*alpha_rows;
				
			ALPHA = m_transpose (init_ALPHA, alpha_columns, alpha_rows);	

			free_array(init_ALPHA);	
			init_ALPHA = ALPHA;
		}

		double* info_params = NULL;
		int info_params_length = (mx_info_params != NULL) ?  mxGetN(mx_info_params)*mxGetM(mx_info_params) : 0;
		if (info_params_length > 0) {
			info_params = double_array(info_params_length);
			memcpy(info_params, mxGetPr(mx_info_params), info_params_length * sizeof(double));

			PRINTF("Using info_params of size %i\n", info_params_length);
		}

		int* mask     = NULL;
		int mask_size = (mx_mask != NULL) ? mxGetM(mx_mask)*mxGetN(mx_mask) : 0;

		if (mask_size > 0) {
			PRINTF("Using mask of size %i  ...\n", mask_size);

			mask = int_array(mask_size);

			double* pMask = mxGetPr(mx_mask);

			double2int(mask, pMask , mask_size);
			issub(mask, mask, 1, mask_size);
		}

		double* B    = NULL;	
		double** UCL = NULL;
		int ccount   = 0;
		int UCL_rows = 0;

		char* state_file = NULL;

		SVMconfig* config = new SVMconfig(l_params, l_params_length, X_columns);

		double nf = (mx_nf != NULL)  ? ((!mxIsEmpty(mx_nf)) ?  *mxGetPr(mx_nf) : 1.0)  : 1.0;

		Kernel* K = Kernel::create_kernel(config->kernel_type(), params, params_length);
		K->SetNormFactor(nf);

		//K->precompute(X_train);
		
		//if (nf == 0) K->ComputeNormFactor(X_train);

		if (K==NULL) {
			PRINTF("Could not interpret kernel parameters ... bailing out.\n");
			return;
		}

		if (mask != NULL) X_train = new ClassInfoDimSelectDataDesc(mask, mask_size, X_train);

		InfoConfig* iconfig = new InfoConfig(config->info_mode(), info_params, info_params_length);	

		HashConfig* hconfig = new HashConfig(hash_func_params, hash_func_params_length);

		SubsetClassInfoDataDesc* r_SVs = NULL;

 		if (!svm_wrapper (&ALPHA, &B, &r_SVs, &UCL, &UCL_rows, &ccount, X_train,  config, K, hconfig, iconfig, learn_index, learn_index_size, state_file)) {

			for (int i=0; i < nlhs; i++) plhs[i] = NULL;
		}
		else {
			int svCount = r_SVs->size();

			PRINTF("ccount: %i,  svCount: %i\n", ccount, svCount);

			double** RETURN_ALPHA = m_transpose(ALPHA, ccount, svCount);
			mxArray* mx_ALPHA = mxCreateNumericMatrix (ccount, svCount, mxDOUBLE_CLASS, mxREAL);	
			m_to_vec(mxGetPr(mx_ALPHA), RETURN_ALPHA, svCount, ccount);

			m_free(ALPHA);
			m_free(RETURN_ALPHA);

			mxArray* mx_B = mxCreateNumericMatrix (1,ccount, mxDOUBLE_CLASS, mxREAL);
			memcpy(mxGetPr(mx_B), B, ccount*sizeof(double)) ;

			free_array(B);

			mxArray* mx_SVs = NULL;
			if (!save_memory) {
				mx_SVs = mxCreateNumericMatrix (X_rows, svCount, mxDOUBLE_CLASS, mxREAL);

				for (int i=0; i < svCount; i++) {
					double* pSV = NULL;
					r_SVs->GetSample(&pSV, i);

					memcpy (mxGetPr(mx_SVs) + i*X_rows, pSV, X_rows * sizeof(double));
				}
			}
			else mx_SVs = mxCreateNumericMatrix (0, 0, mxDOUBLE_CLASS, mxREAL);

			mxArray* mx_cSV = mxCreateNumericMatrix (1, svCount, mxDOUBLE_CLASS, mxREAL);
			r_SVs->getClassInfo(mxGetPr(mx_cSV));

			int ucl_length = (ccount > 1) ? ccount : 2;
			mxArray* mx_UCL = mxCreateNumericMatrix (ucl_length, UCL_rows, mxDOUBLE_CLASS, mxREAL);
			m_to_vec (mxGetPr(mx_UCL), UCL, UCL_rows, ucl_length);
			if (return_info) {
				MIndex info_size = ccount;	
				mxArray* mx_info = mxCreateCellArray(1, &info_size);

				for (int i=0; i < ccount; i++) {

					iconfig->set_classifier(i);

					mxArray* mx_info_i = mxCreateNumericMatrix (iconfig->cur_rows(), iconfig->cur_columns(), 
										    mxDOUBLE_CLASS, mxREAL);
					mxSetCell(mx_info, i, mx_info_i);

					if(iconfig->cur_rows()*iconfig->cur_columns() != 0) {
						m_to_vec(mxGetPr(mx_info_i), iconfig->cur_r_info(), 
								 iconfig->cur_columns(), iconfig->cur_rows());
					}
				}

				plhs[6] = mx_info;
			}

			if (return_sv_indizes) {

				int* sv_indizes = int_array(svCount);

				r_SVs->getAbsIndex(sv_indizes);

				for (int i=0; i < svCount;i++) sv_indizes[i]++; // matlab indices are 1 based

				mxArray* mx_sv_indizes = mxCreateNumericMatrix (1,svCount, mxINT32_CLASS, mxREAL); 

				memcpy (mxGetPr(mx_sv_indizes), sv_indizes, svCount*sizeof(int));
	
				plhs[5] = mx_sv_indizes;

				free_array(sv_indizes);
			}

			delete r_SVs;

			plhs[0] = mx_ALPHA;
			plhs[1] = mx_B;
			plhs[2] = mx_SVs;
			plhs[3] = mx_cSV;
			plhs[4] = mx_UCL;
		}

		delete config;
		delete K;
		delete iconfig;
		delete hconfig;

		free_array(c);
		free_array(l_params);
		free_array(params);

		delete X;
		delete X_train;

		free_array(train_index);
		free_array(learn_index);

		if (info_params) free_array(info_params);
		if (init_ALPHA) {
			free_array (init_ALPHA[0]); // svm_wrapper overwrites ALPHA !!!
			free_array (init_ALPHA);
		}

		if (mask != NULL) free_array(mask);
	}
	else {
		mexPrintf ("Wrong number of input/output parameters (%i/%i). \n", nrhs, nlhs);
		for (int i=0; i < nlhs; i++) plhs[i] = NULL;
	}
}
