#ifndef __CACHE_H_INCLUDED__
#define __CACHE_H_INCLUDED__

/* cache structure for internal use */
typedef struct {
	double** Values;
	int** Keys;

	int* usage;

	int misses;
	int collisions;
	int used;
	int released;

	int size;
	int max_usage;
	int entry_size;
	int key_length;

	double* temp;
	int* key_temp;

	int time_max;

	int* hash_func_params;
	int cache_on;

	int delete_hash_func_params;
} tCache;

/*
*/
void init_cache (tCache* cache,int size, int max_usage, int entry_size, int key_length, int time_max, int* hash_func_params);

/*
*/
void free_cache (tCache* cache);

/*
	return value:
	   if get_cache_entry returns 1, the values belonging to the key were found and can be accessed via *x
	   if get_cache_entry returns 0, the values belonging to the key weren't found but can be stored in *x. Furthermore *x contains  wrong but valid data. *r_key contains the key that describes this data.
	   if get_cache_entry returns -1, the values belonging to the key werent't found but can be stored in *x. *x does not contain valid data. 

	   if the return value does not equal 1 the new key of the data that was stored in *x has to be stored in *r_key.
*/
int get_cache_entry (double** x, int** r_key, int time, tCache* cache);

/*
input:
   X_columns : number of training data	
   k_max : maximum number of iterations
   C : number of training data to be used for averaging
output:
   maximum number of sum cache lines
*/
int calc_cache_size (int X_columns, int k_max, int C);

/*
input:
   X_columns : number of training data	
   k_max : maximum number of iterations
   C : number of training data to be used for averaging
output:
   return value : integer array filled with random data to be used for universal hashing
   *size : size of the random data
*/ 
int* create_uv_hash_params (int X_columns, int k_max, int C, int* size);

void check_hash_func_params (int X_columns, int k_max, int C, int** hash_func_params, int *hash_func_params_length, int* delete_hash_func_params);

#endif
