/*
(Soft)MaxMinOver learning algorithm written by Kai Labusch
Copyright (C) 2004 Kai Labusch

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Contact:
labusch@inb.uni-luebeck.de
*/

#ifndef __SVM_H_INCLUDED__
#define __SVM_H_INCLUDED__

extern const int MMO_PARAMS_SIZE;
extern const int DEFAULT_G_SIZE_MB;
extern const int DEFAULT_S_SIZE_MB;

#include <cstdio>
#include "cache.h"

class Kernel;
class SVMconfig;
class InfoConfig;
class HashConfig;
class ClassInfoDataDesc;
class SubsetClassInfoDataDesc;
class DataDesc;

// svm_wrapper trains a multi class classifier by performing training of a number of two-class classifiers
int svm_wrapper (double*** r_ALPHA, double** r_B, SubsetClassInfoDataDesc** r_SVs, double*** r_UCL, int* UCL_rows, int* ccount, 
		 ClassInfoDataDesc* X,  SVMconfig* config, Kernel* K, HashConfig* hconfig, InfoConfig* iconfig,
		 int* learn_index=NULL, int learn_index_len=0, char* state_file=NULL);

// svm training using the dmmo algorithm 
int svm (double* r_alpha, double* r_b, double* decision,
	 tCache** r_G_cache, tCache** r_S_cache, 
	 ClassInfoDataDesc*X, double* c, int k_start, int ccount, int* learn_index, int learn_index_len, 
	 SVMconfig* config, Kernel* K, HashConfig* hconfig, InfoConfig* iconfig);

// perform classification of unknown data using a SVM that has been trained using svm_wrapper
void svm_classify (double*** r_DECISION, double** r_labels, double** ALPHA, double* B, 
		   DataDesc* SVs, double** UCL, int UCL_rows, int ccount, Kernel* K, DataDesc* X, int output=1);

// rank features according to their importance for the margin
void rank_features_margin (double*** r_RANK, double** ALPHA, double* B, DataDesc* SVs, int ccount, Kernel* K, int output);

// rank features according to their importance for the classification of the samples in X
void rank_features (double*** r_RANK, double** ALPHA, double* B, DataDesc* SVs, double** UCL, int UCL_rows, int ccount,
		   Kernel* K, ClassInfoDataDesc* X);

// perform -1/1 assignment that corresponds to chosen multi-class classifier mode
double** class_assign(double* c, int c_length, double** UN, int UN_rows, int ccount);

// check if classifier configuration is sound with respect to given training data, provide -1/1 two class assignment that corresponds
// to chosen classifier configuration
// return the number of two-class classifiers
int check_class_information (double*** r_CL, double*** r_UN, int* r_UN_rows,  double* c, int c_length, SVMconfig* config);

#endif
