/*
(Soft)MaxMinOver learning algorithm written by Kai Labusch
Copyright (C) 2004 Kai Labusch

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Contact:
labusch@inb.uni-luebeck.de
*/

#define ONE_AGAINST_ALL 0
#define ONE_AGAINST_ONE 1

#define RBF_PARAMS_LENGTH 1
#define LIN_PARAMS_LENGTH 0
#define POLY_PARAMS_LENGTH 2

#define PARAM_COUNT 		11

#define K_MAX_POS 		0
#define KERNEL_TYPE_POS		1
#define C_POS			2
#define MODE_POS		3
#define FORGET_SV_POS		4
#define MULTI_CLASS_STRATEGY_POS 5
#define STOP_CRITERION_POS 	6
#define EPSILON_POS		7
#define INFO_MODE_POS		8
#define G_CACHE_MB_POS		9
#define S_CACHE_MB_POS		10
//#define C2_POS			11

#define ZERO_SET_INFO 	1
#define FORGET_INFO 	2
#define CONV_INFO	3
#define NO_INFO		0

#define GAMMA_POS	0
#define DEGREE_POS	0

#include <cstring>
#include <cmath>

class SVMconfig {
public:
	SVMconfig(void);

	SVMconfig(const SVMconfig* c);

	SVMconfig(double* lparams, int l_params_length, int tr_size);

	~SVMconfig(void);
	
	void Default(void);

	SVMconfig* Copy(void);

	int eval (double* l_params, int l_params_length, int tr_size, int output=1);
	int get  (double* l_params, int l_params_length);

	void set (const char* name, double val, int tr_size);

	void DescInit(void);

	int kernel_type(void);
	int k_max(void); 
	int mode(void);
	int C(void);
	double C2(void);
	double epsilon(void);
	double stop_criterion(void);
	int forget_sv(void);
	int mc_strategy(void);
	int info_mode(void);
	int G_size_MB(void);
	int S_size_MB(void);

	char** ident(void);
	char*  ident_str(void);
	void   ident_str(char* dest);

	virtual void set_classifier(int i);

	int do_output(void);
	void output_on(void);
	void output_off(void);

	virtual double GridMap(const char* name, double val) {
		if(strcmp(name,"C")==0) return pow(10,val);
		return val;
	}

	virtual double GridMin(const char* name, double val) {
		if(strcmp(name,"C")==0) return -2.0;
		return val;
	}

	virtual double GridMax(const char* name, double val) {
		if(strcmp(name,"C")==0) return 2;
		return val;
	}

	virtual double GridStopTol (const char* name, double val, int count) {
		if(strcmp(name,"C")==0) return ((double)count)*0.1;
		return val;
	}

private:

	void set_pointers(void);

	int cur_classifier;
	double* m_kernel_type;
	double* m_k_max;
	double* m_mode;
	double* m_C;
	double* m_C2;
	double* m_forget_sv;
	double* m_mc_strategy;
	double* m_info_mode;
	double* m_G_size_MB;
	double* m_S_size_MB;
	double* m_epsilon;
	double* m_stop_criterion;

	char** m_ident;
	char*  m_ident_str;	

	double** pmap;

	int nconfigs;
	int output;
};

class InfoConfig {
public:
	InfoConfig(int imode, double* iparams=NULL, int iparams_len=0);
	InfoConfig(const InfoConfig* c);
	~InfoConfig(void);

	void Init(int num_samples, int ccount, int k_max);

	void store_info(double* vals, int len, int k);

	void set_classifier(int i);
	
	double** cur_r_info(void);
	int cur_rows(void);
	int cur_columns(void);

	int info_mode(void);

private:
	int m_info_mode;
	double*** r_info;

	int* info_rows;
	int* info_columns;

	double* info_params;
	int info_params_length;	

	int cur_classifier;

	int position;

	int m_ccount;
};

class HashConfig {
public:
	HashConfig(int* hparams=NULL, int hparam_len=0);
	~HashConfig(void);

	int* hash_func_params;
	int  hash_func_params_length;

	int allocated;
};
