%function params = lparam_set (mparams, varargin);
%set learning parameter
%
%Examples:
% p = lparam_set(1); <- p contains the standard learning parameters for the MaxMinOver package
% p = lparam_set(2); <- p contains the standard learning parameters for the OSU-SVM package
% p = lparam_set(1,'k_max', 10000,'forget_sv','off'); <- standard MMO learning parameters/k_max set to 100000 iterations/forgetting off
%
% disp(lparam_get('description', p)); <- display description of p
%
function params = lparam_set (mparams, varargin)

if(nargin == 1) && iscell(mparams),
    params = mparams{1};
    return;
end

if (~iscell(mparams)),
    package = mparams;
    mparams = [];
else
    package = mparams{1};
end

%---------------------

[params, p_names, p_index, c_index, descriptions] = lparam_info(package);


if ~isempty(mparams),
    params = mparams(1:length(params));
end

%---------------------

cell_params = [];

i=1;
while i <= length(varargin),
    
    if ischar(varargin{i}),
    
        pos = find(strcmp(p_names, varargin{i}));
    
        if ~isempty(pos),
            if ~ischar(varargin{i+1}),
                if prod(size(varargin{i+1})) > 1,
                    params{c_index(pos)} = varargin{i+1};
                else
                    params{c_index(pos)}(p_index(pos)) = varargin{i+1};
                end
            else
                desc = descriptions{pos};
                if ~isempty(desc),            
                    p = find(strcmp(desc, varargin{i+1}));
                    
                    if ~isempty(p),
                        params{c_index(pos)}(p_index(pos)) = p - 1;
                    else
                        disp (sprintf ('unknown value %s for parameter %s.', varargin{i+1}, varargin{i}));
                    end
                else
                    disp(sprintf('no descriptions for parameter %s available.', varargin{i+1}));
                end
            end
        else
            disp(sprintf('Unknown parameter : %s', varargin{i}));
        end
        
        i = i + 2;
    else        
        cell_params = [ cell_params, i];
        
        i = i + 1;
    end
end

if ~isempty(cell_params),
   params = { params{1:length(params)} , varargin{cell_params} };
end

return;
