% [X, c, S, IS] = lsep (w , b , count , r , g, t, sd);
% [X,c, S, IS] = lsep (wnorm , b , count, r, g, t, sd, mode, percent);
%
% Generiert eine Menge von (linear separierbaren) n-dimensionalen Vektoren
% Eingabe:
%   w : Zeilenvektor Normalenvektor der separierenden Hyperebene
%   b : Skalar Abstand vom Ursprung (fuer norm(wnorm) == 1)
%   count : Anzahl der Vektoren
%   r : Begrenzung des Bereiches auf der Hyperebene aus dem die Vektoren uniform gewaehlt werden (Hyperquadrat mit Kantenlaenge r und Mittelpunkt 0)
%   g : Gewichtsfaktor fuer die Abweichung der Vektoren von der Hyperebene (Stoerung)
%   t : Mindestabstand der Vektoren von der Trennebene
%   sd : Abstand der Supportvektoren von der Trennebene (maximum margin)
%   mode : mode ~= 0 => Abweichung von der Ebene ist normalverteilt, mode == 0 => Abweichung uniform verteilt
%   percent : gibt an wieviel Prozent der Trainingsvektoren exakt auf den margin gesetzt werden sollen
% Ausgabe :
%   X : Matrix welche die einzelnen Vektoren als Spalten enthaelt
%   c : Klasse zu der der jeweilige Spaltenvektor gehoert (-1,1)
%   S : die Supportvektoren der optimalen Trennebene
%   IS : die Indizes der Supportvektoren
function [X,c, S, IS] = lsep (wnorm , b , count, r, g, t, sd, mode, percent);


if ~exist('mode','var'),
    mode = 0;
end

if ~exist('percent','var'),
    percent = 0;
end

%---------------------------------

b = b*1/norm(wnorm);
wnorm = wnorm*1/norm(wnorm);

w = wnorm (1 : size(wnorm,2) - 1) * -1/wnorm(size(wnorm,2));
b = b * -1/wnorm(size(wnorm,2));

% Vektoren uniform auf der Hyperebene verteilen
XT = (rand (size(w') + [0, count - 1])) * r - (0.5*r); 

% Normalenvektor erzeugen
wn = wnorm * 1/norm(wnorm);

if (mode ~= 0),
    % Gewichtung fuer den Normalenvektor erzeugen (uniformverteilt)
    noise = (1 - 2*rand (1,count)) * g;
else
    % Gewichtung fuer Normalenvektor erzeugen (normalverteilt)
    noise = randn (1,count) * g;
end

% Klassenzugehoerigkeit ergibt sich aus dem Vorzeichen des Rauschens
c = sign(noise);

% Mindestabstand draufaddieren
noise = noise + t*c;

%Indizes der Klasse -1 raussuchen
IN = find(c == -1);
%Indizes der Klasse 1 raussuchen
IP = find(c == 1);

%------------------------------------------
% hier werden fuer den 2d  Fall 3 Supportvektoren so gesetzt,
% dass sie eindeutig die Trennebene mit Normalenvektor w definieren

% Indizes der Supportvektoren aus den beiden Klassen finden
IS = ceil(length(noise) * rand(1,3));

[SY, I] = sort (XT(1,IS));
IS = IS(I);

%die Supportvektoren genau auf die Kante setzen
noise([IS(1), IS(3)]) = t;
c([IS(1), IS(3)]) = 1;

noise(IS(2)) = -t;
c(IS(2)) = -1;

%------------------------------------------

if (percent ~= 0),
    %Indizes der Klasse -1 raussuchen
    IN = find(c == -1);
    %Indizes der Klasse 1 raussuchen
    IP = find(c == 1);
    
    %Anzahl zu setzender Vektoren ausrechnen
    pn = ceil((length(IN)/100) *percent);
    pp = ceil((length(IP)/100) *percent);
    
    %Indizes dieser Vektoren finden
    IP = IP(ceil(rand(1,pp)*(length(IP)-1)));
    IN = IN(ceil(rand(1,pn)*(length(IN)-1)));
    
    %Vektoren auf den margin setzen
    noise(IP) = sd;
    noise(IN) = -sd;
    
    %die soeben auf den margin gesetzten Vektoren zu den Supportvektoren hinzufuegen
    IS = [IS , IP , IN];
end

% die fehlende Komponente der Vektoren berechnen
y = (w * XT) + b;

% Normalenvektor mit Gewichtung multiplizieren
WT = (noise'*wn)';

% alles zusammenfuegen -> [XT ; y] enthaelt nur Vektoren die genau auf der Hyperebene liegen
% diese werden durch die Addition von WT senkrecht von der Hyperebene weg verschoben
X = [ XT ; y ] + WT;

%Supportvektoren raussuchen
S = X(:,IS);

return;