%
%function [rank] = RankFeaturesMargin(alpha, b, SVs, ucl, kernel_type, k_params, nf, save_memory);
%
% Perform margin based feature ranking by removing each feature from the decision function
% and determining its relevance for the margin. The larger rank(i) is the more important the
% corresponding feature is.
%
% In case of linear classification this provides the same feature ranking as one obtains 
% by considering the absolute entries of the normal vector of the decision hyper-plane.
%
%Input:
% alpha       : weights of the support vectors (for each two class problem)
% b           : threshold (for each two class problem)
% SVs         : support vectors
% ucl         : class assignment for each two class problem
% kernel_type : 0 -> linear, 1 -> RBF, 2 -> polynomial
% k_params    : kernel parameters:
% 	        linear     : []
%	        rbf        : [gamma]
%	        polynomial : [degree,k]
%
% nf          : normalization factor (default-> 1.0, might be obtained by CVMaxMinOver)
% save_memory : 0-> off, default 1-> use as few memory as possible 
%
%Return Values:
% rank        : feature ranking
%
function [rank] = RankFeaturesMargin (alpha, b, SVs, ucl, kernel_type, k_params, nf, save_memory);
